/*
    GuiMame
    Copyright (C) 2001 John David Ratliff [guimame@technoplaza.net]

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// RunPipe.cpp: implementation of the CRunPipe class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "GuiMame.h"
#include "RunPipe.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRunPipe::CRunPipe()
{

}

CRunPipe::~CRunPipe()
{

}

bool CRunPipe::openPipe(CEdit *editCtrl, CString path, CString file)
{
	HANDLE readPipe, writePipe;
	PROCESS_INFORMATION processInfo;
	SECURITY_ATTRIBUTES security;
	STARTUPINFO startup;
	char buffer[8192];
	unsigned long int read, available, left, loop;
	int len;
	short int done = FALSE;

	// clear the output box
	editCtrl->SetWindowText("");

	// clear the structures
	memset(&processInfo,0,sizeof(PROCESS_INFORMATION));
	memset(&security,0,sizeof(SECURITY_ATTRIBUTES));
	memset(&startup,0,sizeof(STARTUPINFO));

	// setup the security attributes
	security.nLength = sizeof(SECURITY_ATTRIBUTES);
	security.bInheritHandle = TRUE;

	if (!CreatePipe(&readPipe,&writePipe,&security,0)) {
		AfxMessageBox("Unable to open Pipe to M.A.M.E.");
		return false;
	}

	// setup the process attributes
	startup.cb = sizeof(STARTUPINFO);
	startup.dwFlags = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
	startup.wShowWindow = SW_HIDE;
	startup.hStdOutput = writePipe;
	startup.hStdError = writePipe;

	// create the M.A.M.E. process
	if (!CreateProcess(NULL,(LPTSTR)(LPCTSTR)file,NULL,NULL,TRUE,0,NULL,(LPCTSTR)path,&startup,&processInfo)) {
		AfxMessageBox("Unable to create M.A.M.E. process");

		// close the handles we opened
		CloseHandle(readPipe);
		CloseHandle(writePipe);

		// return FALSE
		return false;
	}

	while (!done) {
		// we've read 0 bytes so far
		read = 0;

		// check to see if our pipe has information for us
		PeekNamedPipe(readPipe,buffer,1,&read,&available,&left);

		// if we need to read some data...
		if (read > 0) {
			ReadFile(readPipe,buffer,8191,&read,NULL);
			buffer[read] = 0;

			// replace undesirable output
			for (loop = 0; loop < read; loop++) {
				switch (buffer[loop]) {
				case 13:
					if (buffer[loop + 1] != 10) {
						// OxD but no OxA
						if (loop > 0) {
							buffer[loop - 1] = 13;
							buffer[loop] = 10;
						}
					}
					break;
				}
			}

			// go to the end of the control
			len = editCtrl->GetWindowTextLength();
			editCtrl->SetSel(len,len);

			// add the text
			editCtrl->ReplaceSel(buffer);

			// go to the bottom
			editCtrl->LineScroll(editCtrl->GetLineCount());
		} else {
			// Did M.A.M.E. exit?

			if (WaitForSingleObject(processInfo.hProcess,0) == WAIT_OBJECT_0) {
				done = TRUE;
			}
		}
	};

	// close the pipes and processes
	CloseHandle(processInfo.hThread);
	CloseHandle(processInfo.hProcess);
	CloseHandle(readPipe);
	CloseHandle(writePipe);

	return TRUE;
}
